# -*- mode: tcl; indent-tabs-mode: nil -*-
# ----------------------------------------------------------------------
#  COMPONENT: geomapviewpoint - holds viewpoint info for a geomap camera
#
# ======================================================================
#  AUTHOR:  Derrick Kearney, Purdue University
#  Copyright (c) 2004-2015  HUBzero Foundation, LLC
#
#  See the file "license.terms" for information on usage and
#  redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
# ======================================================================
package require Itcl
package require BLT

namespace eval Rappture {
    # forward declaration
}


itcl::class Rappture::GeoMapViewpoint {
    constructor {args} {
        # defined below
    }
    destructor {
        # defined below
    }

    public variable x "0.0"
    public variable y "0.0"
    public variable z "0.0"
    public variable heading "0.0"
    public variable pitch "-89"
    public variable distance "1.0"
    public variable srs ""
    public variable vertDatum ""
    public variable label ""
    public variable description ""

    public method import { from }
    public method export { args }
}

# ----------------------------------------------------------------------
# CONSTRUCTOR
# ----------------------------------------------------------------------
itcl::body Rappture::GeoMapViewpoint::constructor {args} {

    if { [llength $args] == 1 && [$args isa Rappture::GeoMapViewpoint]} {
        # if we are given a viewpoint object, copy settings from it
        set varnames [lsearch -regexp -inline -not -all \
            [$args info variable] {::this$}]
        foreach v $varnames {
            set [namespace tail $v] [$args info variable $v -value]
        }
    } else {
        eval configure $args
    }
}


# ----------------------------------------------------------------------
# DESTRUCTOR
# ----------------------------------------------------------------------
itcl::body Rappture::GeoMapViewpoint::destructor {} {
}


# ----------------------------------------------------------------------
# x: get/set x camera coordinate
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::x {

    if {[string is double $x] == 0} {
        error "bad value \"$x\": x should be a double"
    }
}


# ----------------------------------------------------------------------
# y: get/set y camera coordinate
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::y {

    if {[string is double $y] == 0} {
        error "bad value \"$y\": y should be a double"
    }
}


# ----------------------------------------------------------------------
# z: get/set z camera coordinate
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::z {

    if {[string is double $z] == 0} {
        error "bad value \"$z\": z should be a double"
    }
}


# ----------------------------------------------------------------------
# heading: get/set camera heading
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::heading {

    if {[string is double $heading] == 0} {
        error "bad value \"$heading\": heading should be a double"
    }
}


# ----------------------------------------------------------------------
# pitch: get/set camera pitch
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::pitch {

    if {[string is double $pitch] == 0} {
        error "bad value \"$pitch\": pitch should be a double"
    }
}


# ----------------------------------------------------------------------
# distance: get/set camera distance
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::distance {

    if {[string is double $distance] == 0} {
        error "bad value \"$distance\": distance should be a double"
    }
}


# ----------------------------------------------------------------------
# vertDatum: get/set camera vertDatum
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::vertDatum {

    if {[string equal $vertDatum ""]} {
        error "bad value \"$vertDatum\": vertDatum should be a non-empty string"
    }
}


# ----------------------------------------------------------------------
# label: get/set viewpoint label
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::label {

    if {[string equal $label ""]} {
        error "bad value \"$label\": label should be a non-empty string"
    }
}


# ----------------------------------------------------------------------
# description: get/set viewpoint description
# ----------------------------------------------------------------------
itcl::configbody Rappture::GeoMapViewpoint::description {

    if {[string equal $description ""]} {
        error "bad value \"$description\": description should be a non-empty string"
    }
}


# ----------------------------------------------------------------------
# import: clients use this public method to quickly configure the object
#         based on settings from another viewpoint object.
#
# USAGE:
# import <from>
#
# where <from> is another Rappture::GeoMapViewpoint object
#
# ----------------------------------------------------------------------
itcl::body Rappture::GeoMapViewpoint::import {from} {

    if { [llength $from] == 1 } {
        if { [$from isa Rappture::GeoMapViewpoint] } {
            # import settings from another viewpoint object
            set varnames [lsearch -regexp -inline -not -all \
                [$from info variable] {::this$}]
            foreach v $varnames {
                set [namespace tail $v] [$from info variable $v -value]
            }
        } else {
            error "bad value \"$from\": should be a Rappture::GeoMapViewpoint object"
        }
    } else {
        error "wrong # args \"$from\": should be import <from>"
    }
}


# ----------------------------------------------------------------------
# export: clients use this public method to serialize the object to a
#       blt::tree object
#
# USAGE:
# export -format blt_tree
#
# ----------------------------------------------------------------------
itcl::body Rappture::GeoMapViewpoint::export {args} {
    set valids "-format"
    set format "blt_tree"

    while {[llength $args] > 0} {
        set flag [lindex $args 0]
        switch -- $flag {
            "-format" {
                if {[llength $args] > 1} {
                    set format [lindex $args 1]
                    set args [lrange $args 2 end]
                } else {
                    error "wrong number args: should be ?-format <format>?"
                }
            }
            default {
                error "invalid option \"$flag\": should be one of $valids"
            }
        }
    }

    set valids "blt_tree"

    switch -- $format {
        "blt_tree" {
            set t [blt::tree create]
            $t set root \
                label $label \
                description $description \
                x $x \
                y $y \
                z $z \
                heading $heading \
                pitch $pitch \
                distance $distance \
                srs $srs \
                vertDatum $vertDatum
        }
        default {
            error "bad format \"$format\": should be one of $valids"
        }
    }

    return $t
}

